/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/*******************************************************************************
* File Name    : adcmp.c
* Version      : Applilet EZ PL for RL78
* Device(s)    : R5F1056A,R5F1057A,R5F1058A
* Tool-Chain   : CC-RL
* Description  : This file implements the function of A/D Converter.
* Creation Date: 2016/09/14
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "sfr.h"
#include "common.h"
#include "config.h"
#include "int.h"
#include "adcmp.h"

#if defined ANI3_USED
/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/
extern USHORT usgADCR;

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
static USHORT 	usgANI_Value = 0;

/*******************************************************************************
* Function Name: ADC_init
* Description  : Initialize A/D Converter.
* Arguments    : Trg, Ani
* Return Value : none
********************************************************************************/
void ADC_init( UCHAR Trg, UCHAR AnalogIn )
{
	ADCEN = 1;			/* Enables A/D Converter input clock supply */

	/* A/D CONVERTER */
	ADM0 = ADM0_INIT_VAL;
	if ( (SYS_CLOCK_FREQ >= 1) && (SYS_CLOCK_FREQ <= 8) )
	{
		ADM0 |= AD_CONVERSION_CLOCK_2;
	}
	else if ( (SYS_CLOCK_FREQ > 8) && (SYS_CLOCK_FREQ >= 16) )
	{
		ADM0 |= AD_CONVERSION_CLOCK_4;
	}
	else if ( (SYS_CLOCK_FREQ > 16) && (SYS_CLOCK_FREQ >= 24) )
	{
		ADM0 |= AD_CONVERSION_CLOCK_6;
	}
	ADM1 = ADM1_INIT_VAL;
	if ( Trg == TRIGGER_HARDWARE )
	{
		ADM1 |= AD_HARDWARE_ELC_TRIGGER;
	}
	ADM2 = ADM2_INIT_VAL;

	ADS = AnalogIn;

	if ( Trg == TRIGGER_HARDWARE )
	{
		/* Interrupt enable. */
		ADMK = 0;
		ADIF = 0;
	}

	ADCE = 1;			/* A/D Converter Start */
}

/*******************************************************************************
* Function Name: ADC_Get
* Description  : Convert A to D.
* Arguments    : Trg, AnalogIn
* Return Value : none
********************************************************************************/
void ADC_Get( UCHAR Trg, UCHAR AnalogIn )
{
	if (Trg == TRIGGER_SOFTWARE)
	{
		ADIF = 0;
		ADS = AnalogIn;
		ADCS = 1;
		while (!ADIF);
		ADCS = 0;

		usgANI_Value = (SHORT)(ADCR >> 6);
	}
	else
	{
		usgANI_Value = (usgADCR >> 6);
	}
}

/*******************************************************************************
* Function Name: ADC_Bit
* Description  : Get Status by BIT mode.
* Arguments    : Val  ... bit location.
* Return Value : Status.
********************************************************************************/
UCHAR ADC_Bit( UCHAR Val )
{
	return ((usgANI_Value >> Val) & 0x01);

}

/*******************************************************************************
* Function Name: PGA_init
* Description  : Initialize Programmable Gain Amplifier.
* Arguments    : Gain
* Return Value : none
********************************************************************************/
void PGA_init( UCHAR Gain )
{
	UCHAR ucWait;

	/* Enables PGA input clock supply */
	PGA0EN = 1;

	/* 10usec wait */
	for (ucWait = 0; ucWait < SYS_CLOCK_FREQ * 10; ucWait++)
	{
		NOP();
	}

	PGACTL = Gain & 0x03;	/* Set Gain */

	PGAEN = 1;				/* PGA Start */

}

/*******************************************************************************
* Function Name: DAC_init
* Description  : Initialize D/A converter.
* Arguments    : Gain
* Return Value : none
********************************************************************************/
void DAC_init( UCHAR DaOut )
{
	/* Enables D/A converter input clock supply */
	DACEN = 1;

	DACS0 = DaOut;
	DACE0 = 1;
}

/*******************************************************************************
* Function Name: CMP0_init
* Description  : Initialize comparator 0.
* Arguments    : Gain
* Return Value : none
********************************************************************************/
void CMP0_init( UCHAR Mode, UCHAR CmpSel, UCHAR RefSel, UCHAR Filter, UCHAR Edge )
{
	/* Enables comparator input clock supply */
	CMPEN = 1;
	/* Enables PGA input clock supply */
	PGA0EN = 1;

	switch (Mode)
	{
		case CMP_MODE_NORMAL:
			COMPMDR |= COMP0_MODE_NORMAL;
			break;
		case CMP_MODE_WINDOW:
			COMPMDR |= COMP0_MODE_WINDOW;
			break;
		default:
			break;
	}
	COMPISEL = CmpSel;
	if ( CmpSel == IVCMP_SELECTED )
	{
		COMPISEL |= COM0_IVCMP0_SELECTED;
	}
	else if ( CmpSel == PGA_SELECTED )
	{
		COMPISEL |= COM0_PGA_SELECTED;
	}

	switch (RefSel)
	{
		case IVREF0:
		case IVREF0_1:
			COMPISEL |= COM0_IVREF0_SELECTED;
			break;
		case BGRVREF:
			COMPMDR = COM0_BGRVREF_SELECTED;
			break;
		case DA:
			COMPISEL |= COM0_DACOUTPUT_SELECTED;
			break;
		default:
			break;
	}
	if ( Filter != 0 )
	{
		COMPFIR = COMP0_FILTER_FCLK_32;
	}
	switch (Edge)
	{
		case RISING:
			COMPFIR |= COMP0_ONE_EDGE_INTERRUPT;
			COMPFIR |= COMP0_RISING_INTERRUPT;
			break;
		case FALLING:
			COMPFIR |= COMP0_ONE_EDGE_INTERRUPT;
			COMPFIR |= COMP0_FALLING_INTERRUPT;
			break;
		case BOTH:
			COMPFIR |= COMP0_BOTH_EDGES_INTERRUPT;
			break;
		default:
			break;
	}
	if ( Edge != NON )
	{
		/* Interrupt enable. */
		MK2L &= ~0x01;
		IF2L &= ~0x01;
	}
	C0IE = 1;

	C0ENB = 1;			/* comparator 0 Start */
}

/*******************************************************************************
* Function Name: CMP1_init
* Description  : Initialize comparator 0.
* Arguments    : Gain
* Return Value : none
********************************************************************************/
void CMP1_init( UCHAR Mode, UCHAR CmpSel, UCHAR RefSel, UCHAR Filter, UCHAR Edge )
{
	/* Enables comparator input clock supply */
	CMPEN = 1;
	/* Enables PGA input clock supply */
	PGA0EN = 1;

	switch (Mode)
	{
		case CMP_MODE_NORMAL:
			COMPMDR |= COMP1_MODE_NORMAL;
			break;
		case CMP_MODE_WINDOW:
			COMPMDR |= COMP1_MODE_WINDOW;
			break;
		default:
			break;
	}
	if ( RefSel == BGRVREF )
	{
		COMPMDR = COM1_BGRVREF_SELECTED;
	}
	if ( Filter != 0 )
	{
		COMPFIR = COMP1_FILTER_FCLK_32;
	}
	switch (Edge)
	{
		case RISING:
			COMPFIR |= COMP1_ONE_EDGE_INTERRUPT;
			COMPFIR |= COMP1_RISING_INTERRUPT;
			break;
		case FALLING:
			COMPFIR |= COMP1_ONE_EDGE_INTERRUPT;
			COMPFIR |= COMP1_FALLING_INTERRUPT;
			break;
		case BOTH:
			COMPFIR |= COMP1_BOTH_EDGES_INTERRUPT;
			break;
		default:
			break;
	}
	if ( Edge != NON )
	{
		/* Interrupt enable. */
		MK2L &= ~0x02;
		IF2L &= ~0x02;
	}
	C1IE = 1;

	C1ENB = 1;			/* comparator 1 Start */
}

#endif /* end of defined ANI3_USED */
